# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 20010 Akretion LDTA (<http://www.akretion.com>).
#    @author Raphaël Valyi
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from tools.translate import _
from osv import fields, osv
from datetime import datetime
import netsvc

class project_phase(osv.osv):
    _inherit = "project.phase"

    def get_or_create_purchase(self, cr, uid, order, phase, purchases, sale_flow, is_on_service_invoice, product_id, dest_address_id, context):
        supplier_data = self.pool.get('product.product').read(cr, uid, product_id, ['seller_id'])['seller_id']
        if supplier_data:
            supplier_id = supplier_data #FIXME supplier_data[0] ?
        else:
            raise osv.except_osv('No supplier information!', "product %s has no supplier information!" % self.pool.get('product.product').read(cr, uid, product_id, ['name'])['name'])
        if purchases.get((sale_flow, is_on_service_invoice, supplier_id, dest_address_id), False):
            return purchases[(sale_flow, is_on_service_invoice, supplier_id, dest_address_id)]
        else:
            defaults = self.pool.get('purchase.order').default_get(cr, uid, ['company_id', 'fiscal_operation_category_id'], context)
            supplier_data = self.pool.get('purchase.order').onchange_partner_id(cr, uid, [], supplier_id, defaults['company_id'], defaults['fiscal_operation_category_id'])['value']
            partner_address_id = supplier_data['partner_address_id']
            pricelist_id = supplier_data['pricelist_id']
            fiscal_position = supplier_data['fiscal_position']
            warehouse_id = self.pool.get('purchase.order').default_get(cr, uid, ['warehouse_id'], context)['warehouse_id']
            sale_flow_data = self.pool.get('purchase.order').sale_flow_change(cr, uid, [], sale_flow, supplier_id, warehouse_id)['value']
            invoice_method = sale_flow_data.get('invoice_method', 'picking')
            location_id = sale_flow_data['location_id']
            if sale_flow in ['direct_delivery', 'direct_invoice_and_delivery']:
                location_id = phase.project_id.location_id.id

            purchase_id = self.pool.get('purchase.order').create(cr, uid, {
                      'analytic_account_id': phase.project_id.analytic_account_id.id,
                      'partner_id': supplier_id,
                      'partner_address_id': partner_address_id,
                      'pricelist_id': pricelist_id,
                      'sale_flow': sale_flow != 'normal_mto' and sale_flow or 'normal',
                      'location_id': location_id,
                      'invoice_method': invoice_method,
                      'dest_address_id': dest_address_id,
                      'origin': phase.project_id.name,
                      'phase_id': phase.id,
                      'sale_id': order.id,
                      'fiscal_operation_category_id': defaults['fiscal_operation_category_id'],
                      'fiscal_operation_id': supplier_data.get('fiscal_operation_id', False)
                      })
            purchases[(sale_flow, is_on_service_invoice, supplier_id, dest_address_id)] = purchase_id
            return purchase_id


    def get_or_create_picking(self, cr, uid, order, phase, pickings, sale_flow, is_on_service_invoice, context):
        if pickings.get((sale_flow, is_on_service_invoice), False):
            return pickings[(sale_flow, is_on_service_invoice)]
        else:
            if is_on_service_invoice:
                fiscal_operation_category_id = self.pool.get('l10n_br_account.fiscal.operation.category').search(cr, uid, [('code', '=', 'Simples Remessa')])[0] #FIXME: not exactly robust !
                fiscal_operation_id = self.pool.get('stock.picking').onchange_fiscal_operation_category_id(cr, uid, [], order.partner_shipping_id.id, order.company_id.id, fiscal_operation_category_id)['value']['fiscal_operation_id']
            else:
                fiscal_operation_category_id = order.fiscal_operation_category_id.id
                fiscal_operation_id = order.fiscal_operation_id.id
            picking_id = self.pool.get('stock.picking').create(cr, uid, {
                     'name': phase.name,
                     'sale_id': order.id,
                     'address_id': order.partner_shipping_id.id,
                     'note': order.note,
                     'origin': "project " + phase.project_id.name + " - " + phase.name,
                     'type': 'out',
                     'invoice_state': (sale_flow != 'direct_invoice') and '2binvoiced' or 'none',
                     'company_id': phase.project_id.company_id.id,
                     'phase_id': phase.id,
                     'fiscal_operation_category_id': fiscal_operation_category_id,
                     'fiscal_operation_id': fiscal_operation_id,
                     'fiscal_position': order.fiscal_position.id
            })
            pickings[(sale_flow, is_on_service_invoice)] = picking_id
            return picking_id


    def add_order_line(self, cr, uid, order_id, project_product_allocation, phase):
        date_planned = phase.date_start or datetime.now()
        order_data = self.pool.get('purchase.order').read(cr, uid, order_id, ['pricelist_id', 'partner_id', 'fiscal_position', 'fiscal_operation_category_id', 'fiscal_operation_id'])
        pricelist_id = order_data['pricelist_id'][0]
        supplier_id = order_data['partner_id'][0]
        fiscal_position_id = order_data['fiscal_position'] and order_data['fiscal_position'][0] or False
        order_line_data = self.pool.get('purchase.order.line').product_id_change(cr, uid, [], pricelist_id, project_product_allocation.product_id.id, project_product_allocation.product_estimated_qty, False, supplier_id, date_planned, fiscal_position_id , date_planned, False, False, phase.project_id.name + " - " + phase.name, order_data['fiscal_operation_category_id'] and order_data['fiscal_operation_category_id'][0], order_data['fiscal_operation_id'] and order_data['fiscal_operation_id'][0])['value']
        if project_product_allocation.sale_flow == 'direct_delivery':
            account_analytic_id = phase.project_id.analytic_account_id.id
        else:
            account_analytic_id = False
        purchase_order_line_id = self.pool.get('purchase.order.line').create(cr, uid, {
                'product_id': project_product_allocation.product_id.id,
                'order_id': order_id,
                'product_uom': order_line_data['product_uom'],
                'price_unit': order_line_data['price_unit'],
                'name': order_line_data['name'],
                'product_qty': order_line_data['product_qty'],
                'date_planned': order_line_data['date_planned'],
                'account_analytic_id': account_analytic_id
        })
        return purchase_order_line_id

project_phase()



class sale_order(osv.osv):
    _inherit = "sale.order"

    def action_ship_create(self, cr, uid, ids, *args):
        """overriden to make sure we are the first to execute and filter the ids we want"""
        order_without_project = []
        for order in self.browse(cr, uid, ids):
            if order.true_project_id:
                self.pool.get('project.project').action_create_pickings(cr, uid, [order.true_project_id.id], *args)
            else:
                order_without_project.append(order.id)
        return super(sale_order, self).action_ship_create(cr, uid, order_without_project, *args)

sale_order()



class stock_picking(osv.osv):
    _inherit = "stock.picking"


    def action_invoice_create(self, cr, uid, ids, journal_id=False,group=False, type='out_invoice', context=None):
        """ FIXME: horrible override of fat non modular method that re-attach the service to the invoice... TODO should be part of the localization, also see freight management -> global tax in Brazil"""

        if context is None:
            context = {}

        result = super(stock_picking, self).action_invoice_create(cr, uid, ids, journal_id, group, type, context)

#        if not result:
#           return result

        for inv_id in result.values():
            for inv in self.pool.get('account.invoice').browse(cr, uid, [inv_id]):
                to_remove_ids = []
                for line in  inv.invoice_line:
                    if line.product_id and line.product_id.fiscal_type == 'service':
                        to_remove_ids.append(line.id)

                self.pool.get('account.invoice.line').unlink(cr, uid, to_remove_ids)

        return result

stock_picking()
