# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 20010 Akretion LDTA (<http://www.akretion.com>).
#    @author Raphaël Valyi
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from tools.translate import _
from osv import fields, osv
import netsvc


class sale_order(osv.osv):
    _inherit = "sale.order"


    def get_order_product_lines_to_invoice(self, cr, uid, ids, context=None):
        lines = super(sale_order, self).get_order_product_lines_to_invoice(cr, uid, ids, context)
        for order in self.browse(cr, uid, ids):
            for line in order.order_line:
                if line.resource_product_id:
                    phase = line.resource_product_id.phase_id
                    if phase.state not in ('draft', 'cancel'):
                        total_invoiceable_amount = 0
                        for picking in phase.picking_ids:
                            if picking.state == 'done':
                                for move in picking.move_lines:
                                    if move.sale_line_id.id == line.id and move.product_id.fiscal_type == 'product' and ((context.get('fiscal_type', False) == 'service' and move.product_id.is_on_service_invoice) or context.get('fiscal_type', False) == 'product'):
                                        invoice_qty = 0
                                        for inv_line in line.invoice_lines:
                                            if inv_line.invoice_id.journal_id.name != 'NF Simples Remessa':
                                                invoice_qty += inv_line.quantity
                                        sent_quantity = 0
                                        for move in line.move_ids:
                                            if move.state == 'done':
                                                sent_quantity += move.product_uos_qty

                                        delta_quantity = sent_quantity - invoice_qty
                                        if abs(delta_quantity) > 0.01:
                                            if line.resource_product_id.product_corrected_qty: #real required quantity might be different from estimated quantity:
                                                if sent_quantity <= line.resource_product_id.product_corrected_qty:
                                                    invoiceable_amount = delta_quantity * line.price_unit - total_invoiceable_amount - total_invoiceable_amount
                                                else:
                                                    invoiceable_amount = (line.resource_product_id.product_corrected_qty - invoice_qty) * line.price_unit - total_invoiceable_amount
                                                #invoiceable_amount =  line.resource_product_id.product_estimated_qty / line.resource_product_id.product_corrected_qty * delta_quantity * line.price_unit
                                            else:
                                                invoiceable_amount = delta_quantity * line.price_unit
                                            total_invoiceable_amount += invoiceable_amount
                                            if abs(invoiceable_amount) > 0.01:
#TODO                                                if line.is_provided_by_customer:
#                                                     invoiceable_amount = 0
                                                lines.append([line, invoiceable_amount, move.product_id.id])
        return lines



    def manual_invoice_service(self, cr, uid, ids, context=None):
        """service invoice"""
        mod_obj = self.pool.get('ir.model.data')
        wf_service = netsvc.LocalService("workflow")
        inv_ids = set()
        inv_ids1 = set()
        for id in ids:
            for record in self.pool.get('sale.order').browse(cr, uid, id).invoice_ids:
                inv_ids.add(record.id)
        # inv_ids would have old invoices if any
        for id in ids:
            wf_service.trg_validate(uid, 'sale.order', id, 'manual_invoice', cr)
            order = self.browse(cr, uid, id)
            context['fiscal_type'] = 'service'
            created_lines = super(sale_order, self).get_invoice_lines(cr, uid, [id], context)

            if created_lines:
                print "created lines service", created_lines
                res = super(sale_order, self)._make_invoice(cr, uid, order, created_lines, context)
                print "res", res
                cr.execute('insert into sale_order_invoice_rel (order_id,invoice_id) values (%s,%s)', (order.id, res))

            for record in self.pool.get('sale.order').browse(cr, uid, id).invoice_ids:
                inv_ids1.add(record.id)
        inv_ids = list(inv_ids1.difference(inv_ids))

        res = mod_obj.get_object_reference(cr, uid, 'account', 'invoice_form')
        res_id = res and res[1] or False,

        return {
            'name': 'Customer Invoices',
            'view_type': 'form',
            'view_mode': 'form',
            'view_id': [res_id],
            'res_model': 'account.invoice',
            'context': "{'type':'out_invoice', 'fiscal_type': 'service'}",
            'type': 'ir.actions.act_window',
            'nodestroy': True,
            'target': 'current',
            'res_id': inv_ids and inv_ids[0] or False,
        }



    def manual_invoice(self, cr, uid, ids, context=None):
        """By default we admit it's to generate a product invoice"""
        mod_obj = self.pool.get('ir.model.data')
        wf_service = netsvc.LocalService("workflow")
        inv_ids = set()
        inv_ids1 = set()
        for id in ids:
            for record in self.pool.get('sale.order').browse(cr, uid, id).invoice_ids:
                inv_ids.add(record.id)
        # inv_ids would have old invoices if any
        for id in ids:
            wf_service.trg_validate(uid, 'sale.order', id, 'manual_invoice', cr)
            order = self.browse(cr, uid, id)
            context['fiscal_type'] = 'product'
            created_lines = super(sale_order, self).get_invoice_lines(cr, uid, [id], context)
            if created_lines:
                res = super(sale_order, self)._make_invoice(cr, uid, order, created_lines, context)
                cr.execute('insert into sale_order_invoice_rel (order_id,invoice_id) values (%s,%s)', (order.id, res))

            for record in self.pool.get('sale.order').browse(cr, uid, id).invoice_ids:
                inv_ids1.add(record.id)
        inv_ids = list(inv_ids1.difference(inv_ids))

        res = mod_obj.get_object_reference(cr, uid, 'account', 'invoice_form')
        res_id = res and res[1] or False,

        return {
            'name': 'Customer Invoices',
            'view_type': 'form',
            'view_mode': 'form',
            'view_id': [res_id],
            'res_model': 'account.invoice',
            'context': "{'type':'out_invoice'}",
            'type': 'ir.actions.act_window',
            'nodestroy': True,
            'target': 'current',
            'res_id': inv_ids and inv_ids[0] or False,
        }


    def action_invoice_create(self, cr, uid, ids, grouped=False, states=['confirmed', 'done', 'exception'], date_inv = False, context=None):
        create_invoice = False
        for order in self.browse(cr, uid, ids, context=context):
            if (order.order_policy == 'prepaid') or ((order.order_policy == 'postpaid') and order.shipped):
                create_invoice = True

        if create_invoice:
            return super(sale_order, self).action_invoice_create(cr, uid, ids, grouped, states, date_inv, context)
        else:
            return False


        res = False
        invoices = {}
        invoice_ids = []
        picking_obj = self.pool.get('stock.picking')
        invoice = self.pool.get('account.invoice')
        obj_sale_order_line = self.pool.get('sale.order.line')
        if context is None:
            context = {}
        # If date was specified, use it as date invoiced, usefull when invoices are generated this month and put the
        # last day of the last month as invoice date
        if date_inv:
            context['date_inv'] = date_inv
        for o in self.browse(cr, uid, ids, context=context):
            lines = []
            for line in o.order_line:
                if line.invoiced:
                    continue
                elif (line.state in states):
                    lines.append(line.id)
            created_lines = obj_sale_order_line.invoice_line_create(cr, uid, lines)
            if created_lines:
                invoices.setdefault(o.partner_id.id, []).append((o, created_lines))
        if not invoices:
            for o in self.browse(cr, uid, ids, context=context):
                for i in o.invoice_ids:
                    if i.state == 'draft':
                        return i.id
        for val in invoices.values():
            if grouped:
                res = self._make_invoice(cr, uid, val[0][0], reduce(lambda x, y: x + y, [l for o, l in val], []), context=context)
                invoice_ref = ''
                for o, l in val:
                    invoice_ref += o.name + '|'
                    self.write(cr, uid, [o.id], {'state': 'progress'})
                    if o.order_policy == 'picking':
                        picking_obj.write(cr, uid, map(lambda x: x.id, o.picking_ids), {'invoice_state': 'invoiced'})
                    cr.execute('insert into sale_order_invoice_rel (order_id,invoice_id) values (%s,%s)', (o.id, res))
                invoice.write(cr, uid, [res], {'origin': invoice_ref, 'name': invoice_ref})
            else:
                for order, il in val:
                    res = self._make_invoice(cr, uid, order, il, context=context)
                    invoice_ids.append(res)
                    self.write(cr, uid, [order.id], {'state': 'progress'})
                    if order.order_policy == 'picking':
                        picking_obj.write(cr, uid, map(lambda x: x.id, order.picking_ids), {'invoice_state': 'invoiced'})
                    cr.execute('insert into sale_order_invoice_rel (order_id,invoice_id) values (%s,%s)', (order.id, res))
        return res



sale_order()
